#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using System.Collections.Generic;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

/*
changelog:
EdsLevel2Dom_v1.0		revised by Camdo 3/10/2014
- renamed so not confused with origonal work
- added fixed number scaling
- added right margin

EdsLevel2Dom_v2.0:		by origonal author
- combined different depths with same price (good for stocks)
- removed highlighting of 3rd cumulative depth price (complications with combining depths)
- removed clearing DOM when scrolling back, if DOM is out of sync, can just refresh by pressing F5
- realigned position of smaller bars better with chart price
*/

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Speculator Ed's Level 2 indicator inspired from John Thom's jtRealStats indicator.  Press F5 to refresh indicators if level 2 becomes out of sync. edsfreedom.blogspot.com")]
    public class EdsLevel2Dom : Indicator
    {
		public override string ToString()
		{
			return "EdsLevel2Dom_v1.0";
		}
		
        #region Variables
		private	List<DOMRow> askRows = new List<DOMRow>();
		private	List<DOMRow> bidRows = new List<DOMRow>();
		private List<DOMRow> oneDOMRow = null;
		private Dictionary<double, int> askRowsSorted = new Dictionary<double, int>();
		private Dictionary<double, int> bidRowsSorted = new Dictionary<double, int>();
		
		private Font textFont;
		private StringFormat stringFormat;
		private Color textColor = Color.White;
		private Brush textBrush;
		private Color textColor2 = Color.Transparent;
		private Brush textBrush2;
		private Color histAskBarColor = Color.Red;
		private Brush histAskBarBrush;
		private Color histBidBarColor = Color.Blue;
		private Brush histBidBarBrush;
		private Rectangle rect;
		private int x;
		private int y;
		private int totalY;
		private double maxMinusMin;
		private int barLength;
		private int barYOffset;
		private double barLengthScale = 1;
		private int barHeight = 1;
		private bool isCramped = false;
		
		private int histBarOpacity = 10;
		private DateTime lastRefresh = DateTime.MinValue;
		private int refreshDelay = 300;

		private int totalAskVolume = 0;
		private int totalBidVolume = 0;
		private int DOMLargestVolume = 1;
		private int i = 0;
		
		private int rowVolume = 0;
		private int totalRowVolume = 0;
		private string rowVolumeString = "";
		private string totalRowVolumeString = "";
		
		private int marginRt = 1;
		private ScalingSchemeType scalingScheme = ScalingSchemeType.MaximumOfLadder;
		private int scalingNumber = 1000;
		
		public enum ScalingSchemeType
		{
			MaximumOfLadder =0,
			FixedNumber =1,
		}

		
		#endregion

		#region Properties
		[XmlIgnore()]
		[Description("Histogram ask bar color.")]	
		[Category("Level 2 Histogram")]
		[NinjaTrader.Gui.Design.DisplayName("01. Ask Bar Color")]		
		public Color HistogramAskBarColor
		{
			get{ return histAskBarColor; }
			set{ histAskBarColor = value; }
		}
		
		[XmlIgnore()]
		[Description("Histogram bid bar color.")]	
		[Category("Level 2 Histogram")]
		[NinjaTrader.Gui.Design.DisplayName("02. Bid Bar Color")]		
		public Color HistogramBidBarColor
		{
			get{ return histBidBarColor; }
			set{ histBidBarColor = value; }
		}
		
		[XmlIgnore()]
		[Description("Histogram opacity percentage.")]	
		[Category("Level 2 Histogram")]
		[NinjaTrader.Gui.Design.DisplayName("03. Bar Opacity")]		
		public int HistogramOpacity
		{
			get{ return histBarOpacity; }
			set{ histBarOpacity = Math.Max(Math.Min(value, 100), 0); }
		}
		
		[XmlIgnore()]
		[Description("Level 2 refresh delay in milliseconds to avoid overloading CPU.")]	
		[Category("Level 2 Histogram")]
		[NinjaTrader.Gui.Design.DisplayName("04. Refresh Delay")]		
		public int Level2RefreshDelay
		{
			get{ return refreshDelay; }
			set{ refreshDelay = value; }
		}
		
		[Description("Right Margin")]
		[Category("Level 2 Histogram")]
		[NinjaTrader.Gui.Design.DisplayName("05. Right Margin")]		
		public int MarginRt
		{
			get { return marginRt; }
			set { marginRt = Math.Min(2000,Math.Max(0,value)); }
		}
		
		[Description("Scaling Scheme")]
		[Category("Level 2 Histogram")]
		[Gui.Design.DisplayName("06. Scaling Scheme")]

		public ScalingSchemeType ScalingScheme
		{
			get { return scalingScheme;}
			set {scalingScheme = value;}
		}

		[Description("Scaling Number for fixed number scheme")]
		[Category("Level 2 Histogram")]
		[NinjaTrader.Gui.Design.DisplayName("07. ScalingNumber")]		
		public int ScalingNumber
		{
			get { return scalingNumber; }
			set { scalingNumber = Math.Max(1,value); }
		}


        #endregion
		
		#region Initialize
        protected override void Initialize()
        {
            CalculateOnBarClose	= true;
            Overlay				= true;
			DrawOnPricePanel	= true;
			PaintPriceMarkers 	= false;
            PriceTypeSupported	= false;
			PlotsConfigurable 	= false;
			
			textFont = new Font(FontFamily.GenericSansSerif, 12, FontStyle.Regular, GraphicsUnit.Pixel);
			stringFormat = new StringFormat();
			stringFormat.Alignment = StringAlignment.Far;
			textBrush = new SolidBrush(textColor);
			textBrush2 = new SolidBrush(textColor2);
			histAskBarBrush = new SolidBrush(Color.FromArgb(histBarOpacity * 400 / 35, histAskBarColor));
			histBidBarBrush = new SolidBrush(Color.FromArgb(histBarOpacity * 400 / 35, histBidBarColor));
		}
		#endregion
	
		#region On Event Updates
        protected override void OnBarUpdate()
        {
        }
		
		protected override void OnMarketDepth(MarketDepthEventArgs e)
        {
			// Checks to see if the Market Data is of the Ask type
			if (e.MarketDataType == MarketDataType.Ask)
				oneDOMRow = askRows;
			
			// Checks to see if the Market Data is of the Bid type
			else if (e.MarketDataType == MarketDataType.Bid)
				oneDOMRow = bidRows;
			
			if (oneDOMRow == null)
				return;

			// Checks to see if the action taken by the Ask data was an insertion into the ladder
			if (e.Operation == Operation.Insert)
				oneDOMRow.Insert(e.Position, new DOMRow(e.Price, (int) e.Volume));
			
			/* Checks to see if the action taken by the Ask data was a removal of itself from the ladder
			Note: Due to the multi threaded architecture of the NT core, race conditions could occur
			-> check if e.Position is within valid range */
			else if (e.Operation == Operation.Remove && e.Position < oneDOMRow.Count)
				oneDOMRow.RemoveAt(e.Position);
			
			/* Checks to see if the action taken by the Ask data was to update a data already on the ladder
			Note: Due to the multi threaded architecture of the NT core, race conditions could occur
			-> check if e.Position is within valid range */
			else if (e.Operation == Operation.Update && e.Position < oneDOMRow.Count)
			{
				oneDOMRow[e.Position].Price = e.Price;
				oneDOMRow[e.Position].Volume = (int) e.Volume;
			}
			
			try	{
			if (DateTime.Now > lastRefresh.AddMilliseconds(refreshDelay))
			{
				ChartControl.Refresh();
				lastRefresh = DateTime.Now;
			}
			} catch {}
        }
		#endregion
		
		#region Draw Plot
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{	
			// if right-most bar is not a real-time updating bar, then do not display level 2 histogram
			if (ChartControl.LastBarPainted < Bars.Count - 1)
				return;
			
			totalAskVolume = 0;
			totalBidVolume = 0;
			
			// x = ChartControl.CanvasRight;
			x = ChartControl.CanvasRight - marginRt;	//Camdo
			totalY = bounds.Y + bounds.Height;
			maxMinusMin = ChartControl.MaxMinusMin(max, min);

			// find largest volume, copy arrays, and combine depths of same price
			DOMLargestVolume = 1;
			askRowsSorted.Clear();
			bidRowsSorted.Clear();
			for (i = 0; i < askRows.Count; i++)
			{
				if (askRowsSorted.ContainsKey(askRows[i].Price) == false)
					askRowsSorted.Add(askRows[i].Price, askRows[i].Volume);
				else
					askRowsSorted[askRows[i].Price] += askRows[i].Volume;
				if (askRowsSorted[askRows[i].Price] > DOMLargestVolume)
					DOMLargestVolume = askRowsSorted[askRows[i].Price];
			}
			for (i = 0; i < bidRows.Count; i++)
			{
				if (bidRowsSorted.ContainsKey(bidRows[i].Price) == false)
					bidRowsSorted.Add(bidRows[i].Price, bidRows[i].Volume);
				else
					bidRowsSorted[bidRows[i].Price] += bidRows[i].Volume;
				if (bidRowsSorted[bidRows[i].Price] > DOMLargestVolume)
					DOMLargestVolume = bidRowsSorted[bidRows[i].Price];
			}
			
			switch (scalingScheme)
			{
				case ScalingSchemeType.MaximumOfLadder:
					barLengthScale = 120.0 / DOMLargestVolume;
					break;
				case ScalingSchemeType.FixedNumber:
					barLengthScale = 120.0 / scalingNumber;
					break;
				default:
					barLengthScale = 120.0 / DOMLargestVolume;
					break;
			}
			
			// if cramped, draw one histogram bar per two depths
			isCramped = (((totalY - (int) (((min) / maxMinusMin) * bounds.Height)) -
						  (totalY - (int) (((min + TickSize) / maxMinusMin) * bounds.Height))) < 8);

			barHeight = isCramped ? 7 : 9;
			barYOffset = isCramped ? 3 : 5;
			
			foreach (KeyValuePair<double, int> key in askRowsSorted)
			{
				if (key.Value > 0)
				{
					y = totalY - ((int) ((( key.Key - min) / maxMinusMin) * bounds.Height)) - 7;
					totalAskVolume += key.Value;
					barLength = (int) (key.Value * barLengthScale);
					rect = new Rectangle(x - barLength, y + barYOffset, barLength, barHeight);
					graphics.FillRectangle(histAskBarBrush, rect);
					
					if (isCramped == false)
					{
						graphics.DrawString(RowVolumeString(key.Value), textFont, textBrush, x, y, stringFormat);
						graphics.DrawString(RowVolumeString(totalAskVolume), textFont, textBrush2, x - 26, y, stringFormat);			
					}
					else
					{
						if (askRows.Count > 0)
							if ((key.Key == askRows[0].Price) || (key.Key == askRows[askRows.Count - 1].Price))
								graphics.DrawString(RowVolumeString(totalAskVolume), textFont, textBrush, x - 26, y, stringFormat);
					}
				}
			}
			
			foreach (KeyValuePair<double, int> key in bidRowsSorted)
			{
				if (key.Value > 0)
				{
					y = totalY - ((int) ((( key.Key - min) / maxMinusMin) * bounds.Height)) - 7;
					totalBidVolume += key.Value;					
					barLength = (int) (key.Value * barLengthScale);
					rect = new Rectangle(x - barLength, y + barYOffset, barLength, barHeight);
					graphics.FillRectangle(histBidBarBrush, rect);
					
					if (isCramped == false)
					{
						graphics.DrawString(RowVolumeString(key.Value), textFont, textBrush, x, y, stringFormat);
						graphics.DrawString(RowVolumeString(totalBidVolume), textFont, textBrush2, x - 26, y, stringFormat);
					}
					else
					{
						if (bidRows.Count > 0)
						{
							if (key.Key == bidRows[0].Price)
								graphics.DrawString(RowVolumeString(totalBidVolume), textFont, textBrush, x, y, stringFormat);								
							else if (key.Key == bidRows[bidRows.Count - 1].Price)
								graphics.DrawString(RowVolumeString(totalBidVolume), textFont, textBrush, x - 26, y, stringFormat);
						}
					}
				}
			}
		}
		#endregion
		
		#region Support Methods
		private string RowVolumeString(int volume)
		{
			if (volume >= 1000000)
				return (volume / 1000000).ToString() + "M";
			else if (volume >= 100000)
				return (volume / 1000).ToString() + "K";
			else
				return volume.ToString();
		}
		
		private class DOMRow
		{
			public double Price;
			public int Volume;

			public DOMRow(double myPrice, int myVolume)
			{
				Price = myPrice;
				Volume = myVolume;
			}
		}
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private EdsLevel2Dom[] cacheEdsLevel2Dom = null;

        private static EdsLevel2Dom checkEdsLevel2Dom = new EdsLevel2Dom();

        /// <summary>
        /// Speculator Ed's Level 2 indicator inspired from John Thom's jtRealStats indicator.  Press F5 to refresh indicators if level 2 becomes out of sync. edsfreedom.blogspot.com
        /// </summary>
        /// <returns></returns>
        public EdsLevel2Dom EdsLevel2Dom()
        {
            return EdsLevel2Dom(Input);
        }

        /// <summary>
        /// Speculator Ed's Level 2 indicator inspired from John Thom's jtRealStats indicator.  Press F5 to refresh indicators if level 2 becomes out of sync. edsfreedom.blogspot.com
        /// </summary>
        /// <returns></returns>
        public EdsLevel2Dom EdsLevel2Dom(Data.IDataSeries input)
        {
            if (cacheEdsLevel2Dom != null)
                for (int idx = 0; idx < cacheEdsLevel2Dom.Length; idx++)
                    if (cacheEdsLevel2Dom[idx].EqualsInput(input))
                        return cacheEdsLevel2Dom[idx];

            lock (checkEdsLevel2Dom)
            {
                if (cacheEdsLevel2Dom != null)
                    for (int idx = 0; idx < cacheEdsLevel2Dom.Length; idx++)
                        if (cacheEdsLevel2Dom[idx].EqualsInput(input))
                            return cacheEdsLevel2Dom[idx];

                EdsLevel2Dom indicator = new EdsLevel2Dom();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                EdsLevel2Dom[] tmp = new EdsLevel2Dom[cacheEdsLevel2Dom == null ? 1 : cacheEdsLevel2Dom.Length + 1];
                if (cacheEdsLevel2Dom != null)
                    cacheEdsLevel2Dom.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheEdsLevel2Dom = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Speculator Ed's Level 2 indicator inspired from John Thom's jtRealStats indicator.  Press F5 to refresh indicators if level 2 becomes out of sync. edsfreedom.blogspot.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.EdsLevel2Dom EdsLevel2Dom()
        {
            return _indicator.EdsLevel2Dom(Input);
        }

        /// <summary>
        /// Speculator Ed's Level 2 indicator inspired from John Thom's jtRealStats indicator.  Press F5 to refresh indicators if level 2 becomes out of sync. edsfreedom.blogspot.com
        /// </summary>
        /// <returns></returns>
        public Indicator.EdsLevel2Dom EdsLevel2Dom(Data.IDataSeries input)
        {
            return _indicator.EdsLevel2Dom(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Speculator Ed's Level 2 indicator inspired from John Thom's jtRealStats indicator.  Press F5 to refresh indicators if level 2 becomes out of sync. edsfreedom.blogspot.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.EdsLevel2Dom EdsLevel2Dom()
        {
            return _indicator.EdsLevel2Dom(Input);
        }

        /// <summary>
        /// Speculator Ed's Level 2 indicator inspired from John Thom's jtRealStats indicator.  Press F5 to refresh indicators if level 2 becomes out of sync. edsfreedom.blogspot.com
        /// </summary>
        /// <returns></returns>
        public Indicator.EdsLevel2Dom EdsLevel2Dom(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.EdsLevel2Dom(input);
        }
    }
}
#endregion
